
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "bld_html.h"
#include "alblist.h"

static char *Prog = "build_song";	/* program name */
static int debugsw = 0;		/* debug switch */
static int verbose = 0;		/* verbose switch */

void
build_song(char *fname)
{
	char buf[BUFSIZ], buf2[BUFSIZ];
   FILE *outf, *inf;	/* output & input files */
   struct song *song;

   if (fname == NULL) return;
   if (verbose>0) printf("Building song file for %s ...\n",
   	fname);
   if ((song = find_song_by_key(fname)) == NULL)
   {
   	fprintf(stderr, "%s: Invalid song (%s)\n", fname);
      return;
   }

   /*
    * start off by opening the output file
    */
   sprintf(buf, "songs/%s.html", fname);
   if ((outf = fopen(buf, "w")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open output file (%s)\n",
      	Prog, buf);
      return;
   }

   /*
    * next, open the main input file
    */
   sprintf(buf, "sng/%s.html", fname);
   if ((inf = fopen(buf, "r")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open main input file (%s)\n",
      	Prog, buf);
      fclose(outf);	/* close output file */
      return;
   }

   /*
    * We start by creating the header
    * The first line of the main file is the title
    * any non-blank lines following that are for the header
    */

   /* read first line from input file */
   if (read_line(buf2, inf))
   { /* empty file */
   	fprintf(stderr, "%s: input file (%s) was empty\n",
      	Prog, fname);
      return;
   }
   fprintf(outf, "<html><head><title>\nCheryl Wheeler Song %s\n</title>\n", buf2);
   while (1)
   { /* loop copying lines until we find an empty one */
   	if (fgets(buf, BUFSIZ, inf) == NULL) break;
      if (*buf == '\0' || *buf == '\n' || *buf == '\r') break;	/* done */
      fputs(buf, outf);
   }

   /* now create rest of header */
   do_include(outf, "Hdr");

   /* create top level heading */
   fprintf(outf, "<a name=beg><h1>%s</h1></a>\n", buf2);

   bld_html(outf, inf);	/* process this file */
   fclose(inf);	/* close main song file */

   if (song->lyrics)
   { /* lyrics available - create section */
   	do_include(outf, "SongLyrics");
   	bld_lyrics(outf, song);
   }

   /*
    * now, add recording info
    */
   do_include(outf, "SongRecordings");
   bld_songrec(outf, song);

   /* add navigation bar for bottom of page */
   do_include(outf, "songlink");	/* minimsl navigation */

   /*
   * details section complete - add footer
   */
   do_include(outf, "Ftr");

   /* all done - close output file */
   fclose(outf);
}

void
bld_lyrics(FILE *outf, struct song *song)
{ /* create lyrics section */
	FILE *fp;
   char buf[BUFSIZ];
   int eof = 0;

   sprintf(buf, "lyrics/%s.txt", song->key);
   if ((fp = fopen(buf, "r")) == NULL)
   { /* ooops! */
   	fprintf(stderr, "Can't find lyrics file (%s)\n", buf);
      return;
   }

   /* first line is song title */
   if (read_line(buf, fp) != 0)
   { /* EOF */
   	fprintf(stderr, "Empty lyrics file (%s)\n", song->key);
      fclose(fp);
      return;
   }
   do_include(outf, "LyricsTitle");
   fprintf(outf, "%s\n", buf); /* title */
   do_include(outf, "LyricsTitleEnd");
   if (read_line(buf, fp)) eof++; /* ignore blank line */

   /* read lines up to equal signs */
   do_include(outf, "LyricsCredit");
   while (!eof)
   {
   	if (read_line(buf, fp))
      {
	      eof++;
         break;
      }
   	if (*buf == '=') break; /* end of intro */
      if (*buf == '\0') fputs("<br>\n", outf); /* space */
      else fprintf(outf, "<br>%s\n", buf);
   }
   do_include(outf, "LyricsCreditEnd");

   /* now do lyrics until next line w/ equal signs */
   do_include(outf, "LyricsText");
   while (!eof)
   {
   	if (read_line(buf, fp))
      {
      	eof++;
         break;
      }
   	if (*buf == '=') break; /* end of lyrics */
      fputs("<br>", outf);
      if (*buf == '\0') fputs("<p>", outf);
   	else fputs(buf, outf);
      fputc('\n', outf);
   }
   do_include(outf, "LyricsTextEnd");

   /* now the credits until EOF */
	do_include(outf, "LyricsCredit");
   while (!eof)
   {
   	if (read_line(buf, fp)) break;
   	if (*buf == '\0') fputs("<p>", outf);
      else fputs(buf, outf);
      fputc('\n', outf);
   }
   do_include(outf, "LyricsCreditEnd");
   fclose(fp);
}

void
bld_songrec(FILE *outf, struct song *song)
{
   int i;
   struct album *alb;

   if (song->albs->nitems == 0)
   {
   	fprintf(outf, "The song %s doesn't appear on any recordings.\n",
      	song->title);
      return;
   }

   if (song->albs->nitems == 1)
   {
   	alb = find_album_by_key(song->albs->item[0]);
	   fprintf(outf, "The song %s appears on the album\n",
      	song->title);
      if (alb == NULL) fprintf(outf, "[unknown]\n");
      else
      {
      	fprintf(outf, "<a href=../discog/%s.html>%s</a>\n",
      		song->albs->item[0], alb->title);
         if (alb->det != NULL && alb->det->artist != NULL)
         {
      		fprintf(outf, "by %s\n", alb->det->artist);
         }
      }
      return;
   }

   /* song is on more than one album */
   fprintf(outf, "The song %s appears on the following albums:\n<ul>\n",
   	song->title);
	for (i=0; i < song->albs->nitems; i++)
   {
   	alb = find_album_by_key(song->albs->item[i]);
      if (alb == NULL) fprintf(outf, "[unknown]\n");
      else
      {
      	fprintf(outf, "<a href=../discog/%s.html><li>%s</a>\n",
      		song->albs->item[i], alb->title);
         if (alb->det != NULL && alb->det->artist != NULL)
         {
         	fprintf(outf, "by %s\n", alb->det->artist);
         }
      }
   }
}


void
build_all_songs(void)
{
	int i, nsongs;
   char **keys;

   nsongs = get_number_songs();
   keys = get_song_keys();

   for (i=0; i < nsongs; i++) build_song(keys[i]);
}

