/*
 *				**** ALBLIST.H ****
 *
 * This is the header file for the alblist routines
 */

#include "prodlist.h"

#define NUM_SONG_FLDS	5 /* title, clips, lyrics, code, comments */
#define SONG_FLD_TITLE  0
#define SONG_FLD_CLIPS  1
#define SONG_FLD_LYRICS 2
#define SONG_FLD_KEY    3
#define SONG_FLD_COMMENT 4

#define MAX_SONG_ALB  50 /* max. albums for a song */

struct alb_det
{ /* detail album information */
	char *title;	/* album title */
   char *artist;	/* primary artist(s) */
   char *date;		/* date of album */
   char *label;	/* label of album */
   char *catno;	/* catalog number */
   struct alb_det *next;	/* second disc or re-release */
   ProdPtrList *tracks; /* songs on the album */
};

struct album {
	char *key;		/* album code */
   char *title;	/* album title */
   struct alb_det *det; /* album details */
};

struct song {
	char *key;		/* song code */
   char *title;	/* song title */
   int	lyrics;	/* if lyrics are available */
   int	clips;	/* if clips are available */
   char *comments; /* comments */
   ProdAlphaList *albs; /* what albums song appears on */
};

struct albcat {
	char *desc;		/* description of category */
   ProdPtrList *list;	/* list of albums in category */
};

struct track { /* album track */
	char *title;	/* song title */
   char *time;		/* song time */
   char *artist;	/* guest artist */
};


void alblist(char *pname);
void alb_tracks(char *pname);
void songlist(char *pname);
void set_verbose_alblist(int v);
void set_verbose_songlist(int v);
void display_alblist(FILE *fp);
void create_discog(FILE *fp);
void create_discog_links(FILE *fp);
void create_songs(FILE *fp);
void create_songs_links(FILE *fp);

char *get_alb_title_for_key(char *key);
char *get_alb_key_for_title(char *title);
struct album *find_album_by_title(char *title);
struct album *find_album_by_key(char *key);
int get_number_albums(void);
char **get_album_titles(void);
char **get_album_keys(void);
int is_alb_key(char *key);
void build_album(char *fname);
void build_all_albums(void);
void bld_lyrics(FILE *outf, struct song *song);
void bld_songrec(FILE *outf, struct song *song);
struct album *get_album_by_key_loc(int loc);
void update_album_files(char *prog);

struct song *find_song_by_title(char *title);
struct song *find_song_by_key(char *key);
char *get_song_title_for_key(char *key);
char *get_song_key_for_title(char *key);
int get_number_songs(void);
char **get_song_titles(void);
char **get_song_keys(void);
int is_song_key(char *key);
void add_album_to_song(char *albkey, char *songkey);
void display_song_struct(struct song *song);
void display_song_key_ent(int loc);
void display_song_title_ent(int loc);
struct song *get_song_by_key_loc(int loc);
void build_song(char *fname);
void build_all_songs(void);
void song_albums(char *prog);
void write_song_alb(void);
int is_song_alb_updated(struct song *song);
void update_song_files(char *prog);

void read_alb_tracks(char *prog, struct album *alb);
struct alb_det *parse_album(char *str);
struct track *parse_track(char *str);
void bld_alb_det(FILE *fp, char *key);

void do_msg(char *msg[], char *prog);
void print_list(int n, char **list);
int split_flds(char *buf, char *flds[], char delim, int skip, int maxflds);
int read_line(char *buf, FILE *fp);

