
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "bld_html.h"
#include "prodlist.h"
#include "alblist.h"

static char *Prog = "bld_album";		/* program name */
static int debugsw = 0;	/* debug switch */
static int verbose = 0;	/* verbose switch */

void
build_album(char *fname)
{
	char buf[BUFSIZ], buf2[BUFSIZ], *p;
   FILE *outf, *inf;	/* output & input files */

   if (verbose>0) printf("Building album file for %s ...\n",
   	fname);

   /*
    * open the main input file
    */
   sprintf(buf, "alb/%s.html", fname);
   if ((inf = fopen(buf, "r")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open main input file (%s)\n",
      	Prog, buf);
      return;
   }

   /*
    * open the output file
    */
   sprintf(buf, "discog/%s.html", fname);
   if ((outf = fopen(buf, "w")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open output file (%s)\n",
      	Prog, buf);
      fclose(inf);	/* close input file */
      return;
   }

   /*
    * first line is title
    * Remaining lines up to first non-blank
    * are to be placed within the <header>
    * before the >Hdr include segment
    */

   /* read first line from input file */
   if (read_line(buf2, inf))
   { /* empty file */
   	fprintf(stderr, "%s: input file (%s) was empty\n",
      	Prog, fname);
      fclose(outf);
      fclose(inf);
      return;
   }
   fprintf(outf, "<html><head><title>\n%s\n</title>\n", buf2);
   while (1)
   { /* loop copying lines until we find an empty one */
   	if (fgets(buf, BUFSIZ, inf) == NULL) break;
      if (*buf == '\0' || *buf == '\n' || *buf == '\r') break;	/* done */
      fputs(buf, outf);
   }
   do_include(outf, "Hdr");

   /* now build top level header */
   fprintf(outf, "<a name=beg><h1>\n");
   /* check for two line title */
   if ((p = strchr(buf2, '~')) != NULL)
   { /* found a line break */
   	*p++ = '\0';	/* terminate line */
      fprintf(outf, "%s\n<br>%s\n", buf2, p);
   } else fprintf(outf, "%s\n", buf2);
   fprintf(outf, "</h1></a>\n");

   bld_html(outf, inf);	/* process this file */
   fclose(inf);	/* close main album file */

   /*
    * first part of album page done - now the details
    */
   do_include(outf, "AlbMid");

   /* add album details */
   bld_alb_det(outf, fname);

   /* add navigation bar for bottom of page */
   do_include(outf, "alblink0");	/* minimal navigation */

   /*
   * details section complete - add footer
   */
   do_include(outf, "Ftr");

   /* all done - close output file */
   fclose(outf);
}
