/*
 *				**** BLD DISCOG ****
 *
 * This program will build a discography file,
 * along with its link file
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "prodlist.h"
#include "alblist.h"

char *Help[] = {
"This program will build a new discography file\n",
"with its links file.\n",
"The program reads the file alblist.txt, which defines\n",
"all the albums to be included\n",
NULL};

char *Usage[] = {
"Usage:\t%s [options]\n",
"Where options are:\n",
"\t-h\t\tDisplys a help message\n",
"\t-v val\tSet verbose switch to 'val'\n",
NULL};

char *Prog = NULL;	/* program name */
int debugsw = 0;		/* debug switch */
int verbose = 9;		/* verbose switch */

void do_msg(char *msg[], char *prog);
void bld_discog(void);

int
main(int argc, char *argv[])
{
	char buf[BUFSIZ], *p;

   p = strrchr(argv[0], '/');
   if (p == NULL) p = strrchr(argv[0], '\\');
   if (p == NULL) p = argv[0];
   else p++; /* skip past last directory slash */

	Prog = p;	/* save program name */
   /* convert name to lower case & drop .exe (for DOS) */
   while (*p != '\0')
   { /* loop through program name */
   	if (stricmp(p, ".exe") == 0)
      { /* found .EXE (dos) */
      	*p = '\0';
         break;
      }
   	if (isupper(*p)) *p = tolower(*p);
      p++;
   }
   argc--; argv++;	/* skip past program name */

   set_verbose_alblist(verbose);

   while (argc > 0)
   { /* process arguments */
   	if (argv[0][0] == '-') switch(argv[0][1])
      { /* process option */
      	case 'h': do_msg(Usage, Prog); do_msg(Help, Prog); exit(0);
         case 'v':
         	if (argv[0][2] == '\0')
            { /* verbose value is next arg */
            	argc--; argv++;
               p = argv[0];
            } else p = argv[0]+2;
         	verbose = atoi(p);	/* set verbose flag */
            set_verbose_alblist(verbose);
         	break;
         default:
         	fprintf(stderr, "%s: Invalid option (%s)\n",
            	Prog, argv[0]);
            do_msg(Usage, Prog);
            exit(1);
      } else
      { /* not option - must be file name */
      	fprintf(stderr, "%s: file name not processed (%s)\n", argv[0]);
      }
   	argc--; argv++;	/* done with argument */
   }

   bld_discog();	/* build the discography file */

   printf("%s: Terminating\n", Prog);
   exit(0);
}

void
bld_discog()
{
	FILE *inf, *outf;

   /* ingest alblist */
   alblist(Prog);

   /* create the main discog file */
   if ((outf = fopen("work/discog.html", "w")) == NULL)
   {
   	fprintf(stderr, "Can't open output discog file!\n");
   }
   else
   {
   	create_discog(outf);
      fclose(outf);
   }

   /* create discog links file */
   if ((outf = fopen("work/discog_links.html", "w")) == NULL)
   {
   	fprintf(stderr, "Can't open output discog file!\n");
   }
   else
   {
   	create_discog_links(outf);
      fclose(outf);
   }
}

