#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "bld_html.h"
#include "prodlist.h"

static char *Help[] = {
"\nThis program will build HTML pages for the Home directory.\n",
"The program will read files from the work directory,\n",
"merge them with any include segments that are present,\n",
"and write the resulting page to the home directory,\n",
"along with its corresponding links file.\n",
"\n",
"An include segment is signified with a line:\n",
"\t>segname\n",
"where 'segname' is the name of the include segment.\n",
NULL};

static char *Usage[] = {
"Usage:\t%s [options] files ...\n",
"\t%s [-a] [options]\n",
"-OR- for help, type:\n",
"\t%s -h\n",
"where options are:\n"
"\t[-v val]\tset verbose switch to 'val'\n",
"\t-a\t\tprocess all .html files in work\n",
NULL};


char *Prog = NULL;	/* program name */
int debugsw = 0;		/* debug switch */
int verbose = 1;		/* verbose switch */

void do_msg(char *msg[], char *prog);
void bld_home(char *fname);
void bld_all_home();
ProdAlphaList *list_files_in_directory(char *path);

int
main(int argc, char *argv[])
{
	char buf[BUFSIZ], *p;

   p = strrchr(argv[0], '/');
   if (p == NULL) p = strrchr(argv[0], '\\');
   if (p == NULL) p = argv[0];
   else p++; /* skip past last directory slash */

	Prog = p;	/* save program name */
   /* convert name to lower case & drop .exe (for DOS) */
   while (*p != '\0')
   { /* loop through program name */
   	if (stricmp(p, ".exe") == 0)
      { /* found .EXE (dos) */
      	*p = '\0';
         break;
      }
   	if (isupper(*p)) *p = tolower(*p);
      p++;
   }
   argc--; argv++;	/* skip past program name */

   if (argc == 0)
   {
   	do_msg(Usage, Prog);
   	do_msg(Help, Prog);
      exit(0);
   }

   set_verbose_inc(verbose); /* set verbose flag */

   /* parse the include files */
   bld_inc("include/include.txt");

   while (argc > 0)
   { /* process arguments */
   	if (argv[0][0] == '-') switch(argv[0][1])
      { /* process option */
      	case 'a': bld_all_home(); break;
         case 'd': debugsw = !debugsw; break;
      	case 'h': do_msg(Usage, Prog); do_msg(Help, Prog); exit(0);
         case 'v':
         	if (argv[0][2] == '\0')
            { /* verbose value is next arg */
            	argc--; argv++;
               p = argv[0];
            } else p = argv[0]+2;
         	verbose = atoi(p);	/* set verbose flag */
            set_verbose_inc(verbose);
         	break;
         default:
         	fprintf(stderr, "%s: Invalid option (%s)\n",
            	Prog, argv[0]);
            do_msg(Usage, Prog);
            exit(1);
      } else
      { /* not option - must be file name */
      	if (verbose>1) printf("%s: Processing file %s ...\n",
         	Prog, argv[0]);
         strcpy(buf, argv[0]);	/* copy name */

         /* strip .html if there */
         if ((p = strstr(argv[0], ".htm")) != NULL)
         { /* found .htm or .html - get rid of it */
         	*p = '\0';
         }

         /* strip directory name if there */
         p = strrchr(argv[0], '/');
         if (p == NULL) p = strrchr(argv[0], '\\');
         if (p == NULL) p = buf;
         else p++;	/* skip past final slash */
         bld_home(p);	/* build album HTML file */
      }
   	argc--; argv++;	/* done with argument */
   }

   printf("%s: Terminating\n", Prog);
   exit(0);
}

void
bld_home(char *fname)
{
	char buf[BUFSIZ], buf2[BUFSIZ];
   FILE *outf, *inf;	/* output & input files */
   FILE *linkinf, *linkoutf;		/* output file for links */
   char *p;

	if (verbose > 0) printf("Building page %s ...\n", fname);

   /* strip .html if present */
   if ((p = strstr(fname, ".html")) != NULL) *p = '\0';

   /* try to open input file */
   sprintf(buf, "work/%s.html", fname);
   if ((inf = fopen(buf, "r")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open input file (%s)!\n",
      	Prog, buf);
      return;
   }

   /* now try output file */
   sprintf(buf, "home/%s.html", fname);
   if ((outf = fopen(buf, "w")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open output file (%s)!\n",
      	Prog, buf);
      fclose(inf);
      return;
   }

   /*
    * first line is title
    * Remaining lines up to first non-blank
    * are to be placed within the <header>
    * before the >Hdr include segment
    */
	if (read_line(buf2, inf))
   { /* empty file */
   	fprintf(stderr, "%s: input file is empty (%s)\n",
      	Prog, fname);
      fclose(inf);
      fclose(outf);
      return;
   }

   /* write title and whatever follows before blank line */
   fprintf(outf, "<html><head><title>\n%s\n</title>\n", buf2);

   while (1)
   { /* loop copying lines until we find an empty one */
   	if (fgets(buf, BUFSIZ, inf) == NULL) break;
      if (*buf == '\0' || *buf == '\n' || *buf == '\r') break;	/* done */
      fputs(buf, outf);
   }
   do_include(outf, "Hdr");

   fprintf(outf, "<a name=beg><h1>%s</h1></a>\n", buf2);

   /* now process file */
   bld_html(outf, inf);	/* process this file */

   do_include(outf, "Ftr");	/* do footer */

   fclose(outf);

   /* now do no-frames version of the page */
   rewind(inf);	/* rewind to beginning of input file */
   sprintf(buf, "noframe/%s.html", fname);
   if ((outf = fopen(buf, "w")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open output file (%s)!\n",
      	Prog, buf);
      fclose(inf);
      return;
   }

	if (read_line(buf2, inf))
   { /* should never happen - should be caught first time */
   	fprintf(stderr, "%s: input file is empty (%s)\n",
      	Prog, fname);
      fclose(inf);
      fclose(outf);
      return;
   }

   /* write title and whatever follows before blank line */
   fprintf(outf, "<html><head><title>\n%s\n</title>\n", buf2);

   while (1)
   { /* loop copying lines until we find an empty one */
   	if (fgets(buf, BUFSIZ, inf) == NULL) break;
      if (*buf == '\0' || *buf == '\n' || *buf == '\r') break;	/* done */
      fputs(buf, outf);
   }
   do_include(outf, "NoFrameHdr");

   fprintf(outf, "<a name=beg><h1>%s</h1></a>\n", buf2);

   /* now process file */
   bld_html(outf, inf);	/* process this file */

   do_include(outf, "NoFrameFtr");	/* do footer */

   fclose(inf);
   fclose(outf);

   /* Now create the link file */

   /* open input link file */
   sprintf(buf, "work/%s_links.html", fname);
   if ((linkinf = fopen(buf, "r")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open input link file (%s)\n",
      	Prog, buf);
      fclose(inf);
      return;
   }

   /* open output link file */
   sprintf(buf, "home/%s_links.html", fname);
   if ((linkoutf = fopen(buf, "w")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open output link file (%s)\n",
      	Prog, buf);
      fclose(linkinf);
      fclose(outf);
      return;
   }

   if (read_line(buf2, linkinf))
   { /* empty file */
   	fprintf(stderr, "%s: input link file is empty (%s)\n",
      	Prog, fname);
      fclose(linkinf);
      fclose(linkoutf);
   	return;
   }

   fprintf(linkoutf, "<html><head><title>\n%s\n</title>\n", buf2);

   while (1)
   { /* loop copying lines until we find an empty one */
   	if (fgets(buf, BUFSIZ, linkinf) == NULL) break;
      if (*buf == '\0' || *buf == '\n' || *buf == '\r') break;	/* done */
      fputs(buf, linkoutf);
   }

   /* terminate the header and start the body */
   do_include(linkoutf, "Body");

   /* non-blank found - use as page header */
   fprintf(linkoutf, "<a name=beg><font color=#ff0000 size=+2><strong>\n");
   fprintf(linkoutf, "%s</strong></font></a><p>\n", buf2);

   /* now include Hdr segment, & process file */
   bld_html(linkoutf, linkinf);	/* process this file */

   do_include(linkoutf, "BodyEnd");	/* do footer */

   fclose(linkinf);
   fclose(linkoutf);
}

void
bld_all_home()
{
	ProdAlphaList *lst;
   char *p;
   int i;

   if ((lst = list_files_in_directory("work")) == NULL)
   {
   	fprintf(stderr, "%s: No files found in input directory!\n",
      	Prog);
      return;
   }

   for (i=0; i < lst->nitems; i++)
   { /* process all .html files */
   	if ((p = strstr(lst->item[i], ".html")) == NULL) continue;
      /* ignore the links files - they will be build w/ the main page */
      if (strstr(lst->item[i], "_links.html") != NULL) continue;
      *p = '\0'; /* trim .html suffix */
      p = strrchr(lst->item[i], '/');
      if (p == NULL) p = strrchr(lst->item[i], '\\');
      if (p == NULL) p = lst->item[i];
   	bld_home(p);
   }
}

