/*
 *				**** SONG ALBUMS ****
 *
 * This file contains routines to read song/album
 * definition files
 *
 * This file contains a list of each song, along with
 * what albums the songs appear on
 */

 #include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "prodlist.h"
#include "alblist.h"

#define SONG_ALBUM_FILE "tracks/songalb.txt"

static char *Prog="????";

ProdAPtrList *SongAlb=NULL; /* sorted by song key */

void read_song_alb(void);
void write_song_alb(void);

void
song_albums(char *prog)
{
	Prog = prog;

   SongAlb = create_aptr_prodlist(200);
   read_song_alb();
}

void
read_song_alb(void)
{
	char buf[BUFSIZ];
   char *flds[MAX_SONG_ALB];
   int nflds, i;
	ProdAlphaList *list;
   FILE *fp;

   if ((fp = fopen(SONG_ALBUM_FILE, "r")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open song/album file!\n", Prog);
      return;
   }

   while (read_line(buf, fp) == 0)
   { /* read until EOF */
   	if (*buf == '\0') continue; /* blank line */
      nflds = split_flds(buf, flds, '|', 0, MAX_SONG_ALB);
      if (nflds < 1) continue; /* shouldn't happen */
		/* first fields is song, rest are albums */
      list = create_alpha_prodlist(nflds-1);
      for (i=1; i < nflds; i++) list->item[i-1] = strdup(flds[i]);
      list->nitems = nflds - 1;
		bupdate_aptr_prodlist(SongAlb, strdup(flds[0]), list);
   }
   fclose(fp);
}

void
write_song_alb(void)
{ /* create new file using most recent data */
	int nsongs, i, j;
   char **song_keys;
   struct song *song;
   ProdAlphaList *albs;
   FILE *fp;

   if ((fp = fopen(SONG_ALBUM_FILE, "w")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open output song/album file!\n",
      	Prog);
      return;
   }

   nsongs = get_number_songs(); /* how many songs */
   song_keys = get_song_keys(); /* song keys */

   for (i=0; i < nsongs; i++)
   { /* loop thru songs & write entry */
   	song = find_song_by_key(song_keys[i]);
      if (song == NULL) continue; /* shouldn't happen */
   	fputs(song->key, fp); /* write song key */
      albs = (ProdAlphaList *)song->albs; /* albums */
      for (j=0; albs != NULL && j < albs->nitems; j++)
      {
      	fputc('|', fp);
         fputs(albs->item[j], fp);
      }
      fputc('\n', fp); /* end of line */
   }
   fclose(fp);
}

int
is_song_alb_updated(struct song *song)
{ /* return non-zero if entries differ */
	ProdAlphaList *list;
   int loc, i;

   if ((loc = bsearch_aptr_prodlist(SongAlb, song->key)) < 0)
   { /* album not found - must be new */
   	return 1;
   }
   list = (ProdAlphaList *)SongAlb->item[loc]; /* albs */
   /* if different number of entries, it has changed */
   if (list->nitems != song->albs->nitems) return 1;
   /* compare each entry and look for changes */
   for (i=0; i < list->nitems; i++)
   {
   	if (strcmp(list->item[i],song->albs->item[i])) return 1;
   }
   return 0; /* all match */
}
